use crate::{OsDisplay, OsStringFormatter};

macro_rules! impl_borrowed {
    ($type:ty) => {
        impl $crate::OsDisplay for &$type {
            fn fmt_os(&self, f: &mut OsStringFormatter) -> std::fmt::Result {
                (*self).fmt_os(f)
            }
        }

        impl $crate::OsDisplay for &mut $type {
            fn fmt_os(&self, f: &mut OsStringFormatter) -> std::fmt::Result {
                (**self).fmt_os(f)
            }
        }
    };
}
macro_rules! impl_os_from_str {
    ($type:ty) => {
        impl $crate::OsDisplay for $type {
            fn fmt_os(&self, f: &mut OsStringFormatter) -> std::fmt::Result {
                f.write_str(&self.to_string())
            }
        }

        impl_borrowed!($type);
    };
}

impl_os_from_str!(bool);

impl_os_from_str!(u8);
impl_os_from_str!(u16);
impl_os_from_str!(u32);
impl_os_from_str!(u64);
impl_os_from_str!(u128);
impl_os_from_str!(usize);
impl_os_from_str!(i8);
impl_os_from_str!(i16);
impl_os_from_str!(i32);
impl_os_from_str!(i64);
impl_os_from_str!(i128);
impl_os_from_str!(f32);
impl_os_from_str!(f64);

impl_os_from_str!(char);

impl_os_from_str!(&str);
impl_os_from_str!(String);

impl OsDisplay for std::ffi::OsString {
    fn fmt_os(&self, f: &mut OsStringFormatter) -> std::fmt::Result {
        f.write_os_str(self.as_os_str())
    }
}

impl_borrowed!(std::ffi::OsString);

impl OsDisplay for std::ffi::OsStr {
    fn fmt_os(&self, f: &mut OsStringFormatter) -> std::fmt::Result {
        f.write_os_str(self)
    }
}

impl_borrowed!(std::ffi::OsStr);

impl OsDisplay for std::path::Path {
    fn fmt_os(&self, f: &mut OsStringFormatter) -> std::fmt::Result {
        f.write_os_str(&self.as_os_str())
    }
}

impl_borrowed!(std::path::Path);

impl OsDisplay for std::path::PathBuf {
    fn fmt_os(&self, f: &mut OsStringFormatter) -> std::fmt::Result {
        f.write_os_str(&self.as_os_str())
    }
}

impl_borrowed!(std::path::PathBuf);
